#coding:utf-8
"""
Created on 04.12.13

@author: kasimova
"""
import datetime
from dateutil.relativedelta import relativedelta

import settings as gs_settings

from m3.actions import ApplicationLogicException

#---------------------------------------
# Перечисления для формирования запроса
#---------------------------------------
#: Возрастные категории детей

# Способ расчета возраста ребенка
ACADEMIC_YEAR = 1
CALENDAR_YEAR = 0
# Такой способ указан в конфигурации. По-умолчанию это текущий календарный год
CONF_METHOD = gs_settings.AGE_CALCULATION_DATE

def calculate_date():

    today = datetime.date.today()

    def calculate_academic_date():
        """
        Способ расчета возраста на текущий учебный год
        """
        return (datetime.date(today.year, 9, 10) if today >= datetime.date(today.year, 9, 1)
                                             else datetime.date(today.year - 1, 9, 10))

    def calculate_calendar_date():
        """
        Способ расчета возраста на текущий календарный год
        """
        return datetime.date(today.year, 9, 10)

    if CONF_METHOD == CALENDAR_YEAR:
        date = calculate_calendar_date()
    elif CONF_METHOD == ACADEMIC_YEAR:
        date = calculate_academic_date()
    else:
        raise ApplicationLogicException(u'В конфигурациях неверно задан способ расчета возраста ребенка')

    return date

def get_last_day_of_academic_year():
    """
    Последний день текущего учебного года 31.8.год
    """

    today = datetime.date.today()
    today_month = today.month

    if today_month < 9:
        last_day = datetime.date(today.year - 1, 9, 1)
    else:
        last_day = datetime.date(today.year, 9, 1)

    return last_day

def get_report_period():

    today = datetime.date.today()

    beg = datetime.date(today.year, 1, 1)
    end = datetime.date(today.year, today.month, today.day)

    return beg, end

# Дата на которую рассчитывается возраст ребенка.
# По текущему ТЗ это 10.09.текущего_года
# Раснесено по приложению. Поэтому делаю именно так.
CALC_DATE = calculate_date

AGE_DELTAS = {
    '2_months':     CALC_DATE(), # (#90444)
    '6_months':     CALC_DATE() - relativedelta(months=6),
    '1_year':       CALC_DATE() - relativedelta(years=1),
    '1.5_year':     CALC_DATE() - relativedelta(years=1, months=6),
    '2_year':       CALC_DATE() - relativedelta(years=2),
    '2.5_year':     CALC_DATE() - relativedelta(years=2, months=6),
    '3_year':       CALC_DATE() - relativedelta(years=3),
    '3.5_year':     CALC_DATE() - relativedelta(years=3, months=6),
    '4_year':       CALC_DATE() - relativedelta(years=4),
    '4.5_year':     CALC_DATE() - relativedelta(years=4, months=6),
    '5_year':       CALC_DATE() - relativedelta(years=5),
    '5.5_year':     CALC_DATE() - relativedelta(years=5, months=6),
    '6_year':       CALC_DATE() - relativedelta(years=6),
    '6.5_year':     CALC_DATE() - relativedelta(years=6, months=6),
    '7_year':       CALC_DATE() - relativedelta(years=7),
    '7.5_year':     CALC_DATE() - relativedelta(years=7, months=6),
    '8_year':       CALC_DATE() - relativedelta(years=8),
}

AGE_CATEGORIES_FULL = {
    '2-6-MONTHS':   u'от 2 месяцев до 6 месяцев',
    '0.5-1-YEARS':  u'от 6 месяцев до 1 года',
    '1-1.5-YEARS':  u'от 1 года до 1,5 года',
    '1.5-2-YEARS':  u'от 1,5 до 2 лет',
    '2-2.5-YEARS':  u'от 2 до 2,5 лет',
    '2.5-3-YEARS':  u'от 2,5 до 3 лет',
    '3-3.5-YEARS':  u'от 3 до 3,5 лет',
    '3.5-4-YEARS':  u'от 3,5 до 4 лет',
    '4-4.5-YEARS':  u'от 4 до 4,5 лет',
    '4.5-5-YEARS':  u'от 4,5 до 5 лет',
    '5-5.5-YEARS':  u'от 5 до 5,5 лет',
    '5.5-6-YEARS':  u'от 5,5 до 6 лет',
    '6-6.5-YEARS':  u'от 6 до 6,5 лет',
    '6.5-7-YEARS':  u'от 6,5 до 7 лет',
    '7-7.5-YEARS':  u'от 7 до 7,5 лет',
}

# При расчете показателей по очереди было сделано неправильное решение.
# формируется словарик queue_cache[ИД_УЧРЕЖДЕНИЯ][ключ возрастной категории][тип_Учреждения]
# При расчете на 10 сентября и текущюю дату необходимо, чтобы ключи AGE_CATEGORIES_FULL и AGE_CATEGORIES_FULL_CURRENT
# как-то различались.
# НЕ СУДИТЕ МЕНЯ СТРОГО.
AGE_CATEGORIES_FULL_CURRENT = {
    'C2-6-MONTHS':   u'от 2 месяцев до 6 месяцев (на текущую дату)',
    'C0.5-1-YEARS':  u'от 6 месяцев до 1 года (на текущую дату)',
    'C1-1.5-YEARS':  u'от 1 года до 1,5 года (на текущую дату)',
    'C1.5-2-YEARS':  u'от 1,5 до 2 лет (на текущую дату)',
    'C2-2.5-YEARS':  u'от 2 до 2,5 лет (на текущую дату)',
    'C2.5-3-YEARS':  u'от 2,5 до 3 лет (на текущую дату)',
    'C3-3.5-YEARS':  u'от 3 до 3,5 лет (на текущую дату)',
    'C3.5-4-YEARS':  u'от 3,5 до 4 лет (на текущую дату)',
    'C4-4.5-YEARS':  u'от 4 до 4,5 лет (на текущую дату)',
    'C4.5-5-YEARS':  u'от 4,5 до 5 лет (на текущую дату)',
    'C5-5.5-YEARS':  u'от 5 до 5,5 лет (на текущую дату)',
    'C5.5-6-YEARS':  u'от 5,5 до 6 лет (на текущую дату)',
    'C6-6.5-YEARS':  u'от 6 до 6,5 лет (на текущую дату)',
    'C6.5-7-YEARS':  u'от 6,5 до 7 лет (на текущую дату)',
    'C7-7.5-YEARS':  u'от 7 до 7,5 лет (на текущую дату)'
}
AGE_CATEGORIES_FULL_MAP = {
    u'от 2 месяцев до 6 месяцев (на текущую дату)': u'от 2 месяцев до 6 месяцев',
    u'от 6 месяцев до 1 года (на текущую дату)':    u'от 6 месяцев до 1 года',
    u'от 1 года до 1,5 года (на текущую дату)':     u'от 1 года до 1,5 года',
    u'от 1,5 до 2 лет (на текущую дату)':           u'от 1,5 до 2 лет',
    u'от 2 до 2,5 лет (на текущую дату)':           u'от 2 до 2,5 лет',
    u'от 2,5 до 3 лет (на текущую дату)':           u'от 2,5 до 3 лет',
    u'от 3 до 3,5 лет (на текущую дату)':           u'от 3 до 3,5 лет',
    u'от 3,5 до 4 лет (на текущую дату)':           u'от 3,5 до 4 лет',
    u'от 4 до 4,5 лет (на текущую дату)':           u'от 4 до 4,5 лет',
    u'от 4,5 до 5 лет (на текущую дату)':           u'от 4,5 до 5 лет',
    u'от 5 до 5,5 лет (на текущую дату)':           u'от 5 до 5,5 лет',
    u'от 5,5 до 6 лет (на текущую дату)':           u'от 5,5 до 6 лет',
    u'от 6 до 6,5 лет (на текущую дату)':           u'от 6 до 6,5 лет',
    u'от 6,5 до 7 лет (на текущую дату)':           u'от 6,5 до 7 лет',
    u'от 7 до 7,5 лет (на текущую дату)':           u'от 7 до 7,5 лет'
}

#Важен порядок
AGE_CATEGORIES_FULL_LIST = [
    '2-6-MONTHS',
    '0.5-1-YEARS',
    '1-1.5-YEARS',
    '1.5-2-YEARS',
    '2-2.5-YEARS',
    '2.5-3-YEARS',
    '3-3.5-YEARS',
    '3.5-4-YEARS',
    '4-4.5-YEARS',
    '4.5-5-YEARS',
    '5-5.5-YEARS',
    '5.5-6-YEARS',
    '6-6.5-YEARS',
    '6.5-7-YEARS',
    '7-7.5-YEARS',
]
AGE_CATEGORIES_CUT = {
    '2M-3YEARS':    u'от 2 месяцев до 3 лет',
    '3-5-YEARS':    u'от 3 до 5 лет',
    '5-7.5-YEARS':  u'от 5 до 7,5 лет',
}
AGE_CATEGORIES_CUT_LIST = [
    '2M-3YEARS',
    '3-5-YEARS',
    '5-7.5-YEARS'
]

GOVERMENT = u'gov_dou'
NOT_GOVERMENT = u'not_gov_dou'
IP = u'ip_dou'

ALL = u'Все'
NO = u'Нет'

AGE_CATEGORY_DELTAS = {
    ALL:                                (AGE_DELTAS['7.5_year'], CALC_DATE()),
    AGE_CATEGORIES_FULL['2-6-MONTHS']:  (AGE_DELTAS['6_months'], AGE_DELTAS['2_months']),
    AGE_CATEGORIES_FULL['0.5-1-YEARS']: (AGE_DELTAS['1_year'], AGE_DELTAS['6_months']),
    AGE_CATEGORIES_FULL['1-1.5-YEARS']: (AGE_DELTAS['1.5_year'], AGE_DELTAS['1_year']),
    AGE_CATEGORIES_FULL['1.5-2-YEARS']: (AGE_DELTAS['2_year'], AGE_DELTAS['1.5_year']),
    AGE_CATEGORIES_FULL['2-2.5-YEARS']: (AGE_DELTAS['2.5_year'], AGE_DELTAS['2_year']),
    AGE_CATEGORIES_FULL['2.5-3-YEARS']: (AGE_DELTAS['3_year'], AGE_DELTAS['2.5_year']),
    AGE_CATEGORIES_FULL['3-3.5-YEARS']: (AGE_DELTAS['3.5_year'], AGE_DELTAS['3_year']),
    AGE_CATEGORIES_FULL['3.5-4-YEARS']: (AGE_DELTAS['4_year'], AGE_DELTAS['3.5_year']),
    AGE_CATEGORIES_FULL['4-4.5-YEARS']: (AGE_DELTAS['4.5_year'], AGE_DELTAS['4_year']),
    AGE_CATEGORIES_FULL['4.5-5-YEARS']: (AGE_DELTAS['5_year'], AGE_DELTAS['4.5_year']),
    AGE_CATEGORIES_FULL['5-5.5-YEARS']: (AGE_DELTAS['5.5_year'], AGE_DELTAS['5_year']),
    AGE_CATEGORIES_FULL['5.5-6-YEARS']: (AGE_DELTAS['6_year'], AGE_DELTAS['5.5_year']),
    AGE_CATEGORIES_FULL['6-6.5-YEARS']: (AGE_DELTAS['6.5_year'], AGE_DELTAS['6_year']),
    AGE_CATEGORIES_FULL['6.5-7-YEARS']: (AGE_DELTAS['7_year'], AGE_DELTAS['6.5_year']),
    AGE_CATEGORIES_FULL['7-7.5-YEARS']: (AGE_DELTAS['7.5_year'], AGE_DELTAS['7_year']),

    AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS']:  (datetime.date.today() - relativedelta(months=6),
                                                  datetime.date.today()),
    AGE_CATEGORIES_FULL_CURRENT['C0.5-1-YEARS']: (datetime.date.today() - relativedelta(years=1),
                                                  datetime.date.today() - relativedelta(months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C1-1.5-YEARS']: (datetime.date.today() - relativedelta(years=1, months=6),
                                                  datetime.date.today() - relativedelta(years=1)),
    AGE_CATEGORIES_FULL_CURRENT['C1.5-2-YEARS']: (datetime.date.today() - relativedelta(years=2),
                                                  datetime.date.today() - relativedelta(years=1, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C2-2.5-YEARS']: (datetime.date.today() - relativedelta(years=2, months=6),
                                                  datetime.date.today() - relativedelta(years=2)),
    AGE_CATEGORIES_FULL_CURRENT['C2.5-3-YEARS']: (datetime.date.today() - relativedelta(years=3),
                                                  datetime.date.today() - relativedelta(years=2, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C3-3.5-YEARS']: (datetime.date.today() - relativedelta(years=3, months=6),
                                                  datetime.date.today() - relativedelta(years=3)),
    AGE_CATEGORIES_FULL_CURRENT['C3.5-4-YEARS']: (datetime.date.today() - relativedelta(years=4),
                                                  datetime.date.today() - relativedelta(years=3, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C4-4.5-YEARS']: (datetime.date.today() - relativedelta(years=4, months=6),
                                                  datetime.date.today() - relativedelta(years=4)),
    AGE_CATEGORIES_FULL_CURRENT['C4.5-5-YEARS']: (datetime.date.today() - relativedelta(years=5),
                                                  datetime.date.today() - relativedelta(years=4, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C5-5.5-YEARS']: (datetime.date.today() - relativedelta(years=5, months=6),
                                                  datetime.date.today() - relativedelta(years=5)),
    AGE_CATEGORIES_FULL_CURRENT['C5.5-6-YEARS']: (datetime.date.today() - relativedelta(years=6),
                                                  datetime.date.today() - relativedelta(years=5, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C6-6.5-YEARS']: (datetime.date.today() - relativedelta(years=6, months=6),
                                                  datetime.date.today() - relativedelta(years=6)),
    AGE_CATEGORIES_FULL_CURRENT['C6.5-7-YEARS']: (datetime.date.today() - relativedelta(years=7),
                                                  datetime.date.today() - relativedelta(years=6, months=6)),
    AGE_CATEGORIES_FULL_CURRENT['C7-7.5-YEARS']: (datetime.date.today() - relativedelta(years=7, months=6),
                                                  datetime.date.today() - relativedelta(years=7)),

    AGE_CATEGORIES_CUT['2M-3YEARS']: (AGE_DELTAS['3_year'], AGE_DELTAS['2_months']),
    AGE_CATEGORIES_CUT['3-5-YEARS']: (AGE_DELTAS['5_year'], AGE_DELTAS['3_year']),
    AGE_CATEGORIES_CUT['5-7.5-YEARS']: (AGE_DELTAS['7.5_year'], AGE_DELTAS['5_year']),
}

#: Вид подачи заявления
DELIVERY_TYPES = {
    'IN_PERSON':    u'При личном обращении',
    'PORTAL':       u'С портала'
}
#: Виды ДОО
DOO_TYPES = {
    'NON_STATE':    u'Негосударственные',
    'BUSINESS':     u'ИП'
}
AGE_CATEGORIES_CALC = {
    #key: (age1, age2, age1_month, age2_month)
    'ALL':           (0, 7, 0, 6), # (#90444)
    '2-6-MONTHS':    (0, 0, 0, 6), # (#90444)
    '0.5-1-YEARS':   (0, 1, 6, 0),
    '1-1.5-YEARS':   (1, 1, 0, 6),
    '1.5-2-YEARS':   (1, 2, 6, 0),
    '2-2.5-YEARS':   (2, 2, 0, 6),
    '2.5-3-YEARS':   (2, 3, 6, 0),
    '3-3.5-YEARS':   (3, 3, 0, 6),
    '3.5-4-YEARS':   (3, 4, 6, 0),
    '4-4.5-YEARS':   (4, 4, 0, 6),
    '4.5-5-YEARS':   (4, 5, 6, 0),
    '5-5.5-YEARS':   (5, 5, 0, 6),
    '5.5-6-YEARS':   (5, 6, 6, 0),
    '6-6.5-YEARS':   (6, 6, 0, 6),
    '6.5-7-YEARS':   (6, 7, 6, 0),
    '7-7.5-YEARS':   (7, 7, 0, 6),
    '1-2-YEARS':     (1, 2, 0, 0),
    '1-8-YEARS':     (1, 8, 0, 0),

    'C2-6-MONTHS':    (0, 0, 0, 6), # (#90444)
    'C0.5-1-YEARS':   (0, 1, 6, 0),
    'C1-1.5-YEARS':   (1, 1, 0, 6),
    'C1.5-2-YEARS':   (1, 2, 6, 0),
    'C2-2.5-YEARS':   (2, 2, 0, 6),
    'C2.5-3-YEARS':   (2, 3, 6, 0),
    'C3-3.5-YEARS':   (3, 3, 0, 6),
    'C3.5-4-YEARS':   (3, 4, 6, 0),
    'C4-4.5-YEARS':   (4, 4, 0, 6),
    'C4.5-5-YEARS':   (4, 5, 6, 0),
    'C5-5.5-YEARS':   (5, 5, 0, 6),
    'C5.5-6-YEARS':   (5, 6, 6, 0),
    'C6-6.5-YEARS':   (6, 6, 0, 6),
    'C6.5-7-YEARS':   (6, 7, 6, 0),
    'C7-7.5-YEARS':   (7, 7, 0, 6),

    '2M-3YEARS':     (0, 3, 2, 0),
    '3-5-YEARS':     (3, 5, 0, 0),
    '5-7.5-YEARS':   (5, 7, 0, 6),

    '0-7.5-YEARS':   (0, 7, 0, 6),
    '3-7.5-YEARS':   (3, 7, 0, 6)

}

#: Статусы заявления
APPLICATION_STATUSES = {
    'IN_LINE':                  u'В очереди',
    'IN_LINE_DEFERRED':         u'В очереди (отложенный спрос)',
    'ENROLLED':                 u'Зачислены',
    'WANT_CHANGE_DOO':          u'Желающие сменить ДОО',
    'IN_LINE_ENROLLED':         u'В очереди и зачислены',
    'IN_LINE_WANT_CHANGE_DOO':  u'В очереди и желающие сменить ДОО',
    'DENIED':                   u'Отказано в услуге',
}
#: Виды льгот
BENEFIT_TYPES = {
    'HAVE_BENEFITS':  u'Имеются льготы',
    'FEDERAL':      u'Федеральная льгота',
    'REGIONAL':     u'Региональная льгота',
    'MUNICIPAL':    u'Муниципальная льгота'
}
#: Потребности по здоровью
HEALTH_NEEDS = {
    'DISABLED': u'Ограниченные возможности здоровья',
    'HEALTH':   u'Нуждающиеся в оздоровительных группах',
    'CORRECT':  u'Нуждающиеся в компенсирующих группах',
}
#: Виды групп
GROUP_TYPES = {
    'FAMILY':       u'Семейные дошкольные группы',
    'SHORT_STAY':   u'Группы кратковременного пребывания',
    'DISABLED':     u'Группы для детей с ограниченными возможностями здоровья',
    'CORRECT':      u'Компенсирующие группы',
    'HEALTH':       u'Оздоровительные группы',
    'CARE':         u'Группы по присмотру и уходу',
}
#: Федеральные объекты
FEDERAL_DISTRICTS = {
    'CENTRAL_REGION':           u'Центральный федеральный округ',
    'SOUTH_REGION':             u'Южный федеральный округ',
    'NORTHWEST_REGION':         u'Северо-Западный федеральный округ',
    'FAR_EASTERN_REGION':       u'Дальневосточный федеральный округ',
    'SIBERIAN_REGION':          u'Сибирский федеральный округ',
    'URALIAN_REGION':           u'Уральский федеральный округ',
    'VOLGA_REGION':             u'Приволжский федеральный округ',
    'NORTH_CAUCASUS_REGION':    u'Северо-Кавказский федеральный округ'
}

CURRENT_DATE_CALCULATION = u'Дата отчёта'
TEN_SEMPTEMBER = u'10 сентября текущего календарного года'

# Карта соотношений возрастных подкатегорий системы и возрастных категорий ФО
AGE_SUBCATEGORY_MAP = {
    AGE_CATEGORIES_CUT['2M-3YEARS']: [u'0-1', u'1-2', u'2-3', u'1-3', u'1-4'],
    AGE_CATEGORIES_CUT['3-5-YEARS']: [u'3-4', u'4-5', u'3-5', u'4-6', u'3-6',
                                      u'3-7', u'1-5', u'2-5', u'1-6', u'2-4', u'2-6'],
    AGE_CATEGORIES_CUT['5-7.5-YEARS']: [u'5-6', u'6-7', u'5-7', u'4-7', u'2-7', u'1-7']

}
# Карта соотношений возрастных категорий системы и возрастных категорий ФО
AGE_CATEGORY_MAP = {
    AGE_CATEGORIES_CUT['2M-3YEARS']: [u'0-1', u'1-3'],
    AGE_CATEGORIES_CUT['3-5-YEARS']: [u'3-7'],
    AGE_CATEGORIES_CUT['5-7.5-YEARS']: [u'1-7']
}
