# coding: utf-8
"""
Модуль UI элементов приложения.
"""
__author__ = 'fattakhov'
from m3.actions import urls, ControllerCache
from m3_ext.ui import all_components as ext
from kinder.core.forms import BaseListWindow

class ReportFormsListWindow(BaseListWindow):
    """
    Окно списка форм 'Федеральная отчетность'
    """

    def __init__(self, *a, **kw):
        super(ReportFormsListWindow, self).__init__(*a, **kw)
        self.grid_initialized = False
        self.template_globals = 'ui-js/report-forms-list-window.js'
        report_form_ap = ControllerCache.find_pack(
            'gisdo.actions.ReportFormActionPack')
        self.build_window_url = report_form_ap.build_window_url
        self.view_window_url = report_form_ap.view_window_url
        self.settings_window_url = report_form_ap.settings_window_url
        self.read_only = True

    def init_grid_components(self):
        super(ReportFormsListWindow, self).init_grid_components()
        self.grid.handler_dblclick = 'gridDblClickHandler'
        if not self.grid_initialized:
            self.grid_initialized = True
            self.grid.top_bar._items = [
                ext.ExtButton(text=u'Собрать отчет', handler='sendReportTask', icon_cls='icon-clock-play'),
                ext.ExtButton(text=u'Удалить', handler='deleteValueGrid', icon_cls='icon-report-delete'),
                ext.ExtButton(text=u'Обновить', handler='refreshGridStore', icon_cls='table_refresh'),
                ext.ExtButton(text=u'Настройки', handler='settingsWindow', icon_cls='icon-table-gear')]


class ReportFormBuildWindow(ext.ExtEditWindow):
    """
    Окно ввода параметров отчета
    """

    def __init__(self, pack, *args, **kwargs):
        super(ReportFormBuildWindow, self).__init__(*args, **kwargs)
        self.width, self.height = 400, 120
        self.title = u"Параметры сборки отчёта"
        self.modal = True

        self.minimizable = True
        self.form = ext.ExtForm()
        self.form.url = pack.save_tree_url

        self.unit = ext.ExtDictSelectField(
            anchor='100%',
            label=u'Учреждение',
            name='unit_id',
            allow_blank=False,
            hide_edit_trigger=True,
            hide_trigger=True,
            pack=pack.gisdo_pack.__class__
        )
        self.form.items.extend([self.unit])
        self.buttons.extend((
            ext.ExtButton(text=u"Запустить сборку",
                          handler="submitForm",
                          icon_cls='icon-clock-play'),
            ext.ExtButton(text=u"Отмена",
                          handler="cancelForm"),
        ))


class ReportFormViewWindow(ext.ExtWindow):
    """
    Просмотр данных отчета
    """

    def __init__(self, pack, *args, **kwargs):
        """
        @param : pack инстанспака фед отчетности
        @param : unit_pack иестанс пака реестра учреждений с доп. фильтрацией
        """

        super(ReportFormViewWindow, self).__init__(*args, **kwargs)
        self.title = u'Показатели'
        self.maximized = True
        self.layout = 'fit'
        self.template_globals = 'ui-js/report-form-view-window.js'
        self.report_form_row_data_url = (pack.report_form_row_data_action
                                         .get_absolute_url())
        self.excel_url = pack.excel_url
        self.send_url = pack.send_url
        self.panel = ext.ExtPanel(layout='border')
        self.units_tree = ext.ExtTree(allow_container_drop=False,
                                      region='west',
                                      url=pack.gisdo_pack.nodes_action.get_absolute_url())
        self.units_tree.add_column(header=u'Выберите учреждение',
                                   data_index='name',
                                   sortable=False)

        self.data_panel = ext.ExtTabPanel(region='center', auto_width=False)

        grid_attr = {
            'applications': u'Информация о заявлениях',
            'queues':       u'Информация о детях в очереди',
            'enrolled':     u'Информация о зачисленных детях',
            'capacities':   u'Информация о свободных местах'
        }
        for key, val in grid_attr.items():
            panel = ext.ExtPanel(layout='fit', title=val, header=True)
            setattr(self, '%s_grid' % key, ext.ExtGrid(anchor='100%', name=key, height=300))
            setattr(self, '%s_store' % key, ext.ExtDataStore())
            grid = getattr(self, '%s_grid' % key)
            grid.add_column(header=u'Показатель', data_index='name')
            grid.add_column(header=u'Количество', data_index='count')
            grid.set_store(getattr(self, '%s_store' % key))
            panel.items.append(grid)
            self.data_panel.items.append(panel)

        self.panel.items.extend([self.units_tree, self.data_panel])
        self.footer_bar = ext.ExtToolBar()
        self.save_xls_btn = ext.ExtButton(text=u'Сохранить в Excel файле',
                                          handler='saveInExcel',
                                          icon_cls='icon-page-excel')
        self.send_btn = ext.ExtButton(text=u'Отправить отчет',
                                      handler='sendReport')
        self.footer_bar.items.extend([self.save_xls_btn, self.send_btn])
        self.items.append(self.panel)


class ReportFormSettingsWindow(ext.ExtEditWindow):
    """
    Окно редактирования настроек
    """

    def __init__(self, create_new=False,  *args, **kwargs):
        super(ReportFormSettingsWindow, self).__init__(*args, **kwargs)
        self.template_globals = 'ui-js/gisdo-settings-window.js'
        self.create_new = create_new
        self.width, self.height = 400, 340
        self.title = u"Настройки сборки отчёта"
        self.modal = True

        self.minimizable = True
        self.form = ext.ExtForm()
        self.form.url = (ControllerCache.find_pack(
            'gisdo.actions.ReportFormActionPack').save_tree_url)

        self.id = ext.ExtHiddenField(name='id')

        self.time = ext.ExtTimeField(
            anchor='100%',
            allow_blank=False,
            label=u'Время сборки отчета (ежедневно)',
            name='time',
            format='H:i')

        self.is_active = ext.ExtCheckBox(
            label=u'Автосборка',
            box_label=u'Включить автоматический сбор показателей',
            name='is_active')

        self.push_login = ext.ExtStringField(
            anchor='100%',
            label=u'Логин',
            name='push_login',
            allow_blank=False)

        self.push_password = ext.ExtStringField(
            anchor='100%',
            label=u'Пароль',
            name='push_password',
            allow_blank=False)

        self.resend_count = ext.ExtNumberField(
            anchor='100%',
            label=u'Повторы отправки при неудаче',
            name='resend_count',
            allow_decimals=False,
            allow_negative=False,
            max_value=5,
            min_value=1,
            allow_blank=False)

        self.zip_xml = ext.ExtCheckBox(
            label=u'Сжатие',
            box_label=u'Включить сжатие в zip при отправке',
            name='zip_xml')

        self.async_send = ext.ExtCheckBox(
            label=u'Асинхронная отправка',
            box_label=u'Асинхронная отправка данных',
            name='async_send',
            hidden=True
        )

        self.form.items.extend([self.id,
                                self.time,
                                self.is_active,
                                self.resend_count,
                                self.zip_xml,
                                self.async_send,
                                self.push_login,
                                self.push_password])
        self.buttons.extend((
            ext.ExtButton(text=u"Сохранить",
                          handler="submitForm"),
            ext.ExtButton(text=u"Отмена",
                          handler="cancelForm")))


class GisdoUnitEditWindowBuilder(object):
    """
    Переопределен для реализации дополнительный контролов, специфичных для gisdo
    """

    @staticmethod
    def build_extension(parent_win):

        parent_win.template_globals = 'ui-js/gisdo-unit-edit-window.js'

        not_on_federal_report = ext.ExtCheckBox(
            label=u'Не отправлять фед.отчетность',
            name='not_on_federal_report',
            hidden=True
        )

        parent_win.not_on_federal_report = not_on_federal_report
        parent_win.form.items.append(not_on_federal_report)

        return parent_win